;==================================================================================
;
; description: build a cordless phone scanner from your 'Uher CT1 Plus'
; target: AT90S2313/MB1502 PLL
; copyright (c) 2003 by abuserz
;
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
;==================================================================================

.nolist
.include "2313def.inc"
.list

.def var1=r16
.def var2=r17
.def chan=r18

.equ pll_dat=2
.equ pll_clk=3
.equ pll_ena=4
.equ lockled=5
.equ scankey=6

rjmp RESET

RESET:
	ldi var1,RAMEND					;stackpointer initialisieren
	out SPL,var1

    	ldi var1,0b00111100				;1=ausgang 0=eingang
    	out DDRB,var1
    	ldi var1,0b01000000	
	out PORTB,var1					;pullup an portb.6 aktivieren
	
	ldi var1,0b10001000				;referenzteiler schreiben
	ldi var2,8					;fvco = [(P x N) + A] x fosc / R
	rcall PLL_WRITE					;P=64
	ldi var1,0b00000001				;R=1024
	ldi var2,8					;fosc=12800
	rcall PLL_WRITE

	rcall PLL_ENABLE				;pll aktivieren

	ldi chan,17					;channel auf 17 setzen (kanal1)

LOCK:
	sbi PORTB,lockled				;LED einschalten um LOCK anzuzeigen
	sbis PINB,scankey				;wenn die scantaste gedrckt wird,
	rjmp SCAN					;nchsten kanal anspringen
LOOP:
	sbic ACSR,ACO					;wenn RSSI niedrig, nchster kanal
	rjmp LOCK					;sonst auf diesem kanal bleiben
SCAN:
	cbi PORTB,lockled				;LED wieder ausschalten

	inc chan					;channel um 2 erhhen, step=12,5kHz
	inc chan					;kanalabstand aber 25kHz
R1L:cpi chan,15						;wenn channel=15 setzen auf 239
	brbc 1,R1H					;sonst berspringen
	ldi chan,239
R1H:cpi chan,81						;wenn channel=81 setzen auf 145
	brbc 1,R2L					;sonst berspringen
	ldi chan,145

R2L:cpi chan,143					;wenn channel=143 setzen auf 79
	brbc 1,R2H					;sonst berspringen
	ldi chan,79
R2H:cpi chan,241					;wenn channel=241 setzen auf 17
	brbc 1,ROK					;sonst berspringen
	ldi chan,17
ROK:

	ldi var1,0b10000000				;nur [0b10] wird geschrieben
	ldi var2,2
	rcall PLL_WRITE
	ldi var1,0b00101001				;0b00101001 wird geschrieben
	ldi var2,8
	rcall PLL_WRITE
	mov var1,chan					;channel schreiben
	ldi var2,8
	rcall PLL_WRITE
	ldi var1,0b00000000				;nur [0b0] wird geschrieben
	ldi var2,1
	rcall PLL_WRITE
	
	rcall PLL_ENABLE				;pll aktivieren

	rcall SLEEP_25ms				;warten (hier evtl. optimieren)

	rjmp LOOP

;==================================================================================
;serielle programmierung (Fujitsu MB1502)
;
;datenbyte in var1, bytezhler in var2
;MSB wird zuerst geshifted
;==================================================================================
PLL_WRITE:
	cbi PORTB,pll_clk				;startsituation herstellen
	cbi PORTB,pll_dat
	cbi PORTB,pll_ena
PLL_CYCLE:
	rol var1					;register durch carry schieben
	brcs PLL_DATA_UP				;wenn carry=1, pll_dat=1
	cbi PORTB,pll_dat				;sonst pll_dat=0
	rjmp PLL_DATA_DN				;berspringe setzen von pll_dat
PLL_DATA_UP:
	sbi PORTB,pll_dat				;pll_dat setzen
PLL_DATA_DN:
	rcall PLL_DELAY
	sbi PORTB,pll_clk				;takt erzeugen
	rcall PLL_DELAY
	cbi PORTB,pll_clk
	rcall PLL_DELAY
	dec var2					;bytezhler=bytezhler-1
	brne PLL_CYCLE					;schleife verlassen wenn bytezhler=0
	ret

PLL_ENABLE:
	sbi PORTB,pll_ena				;pll aktivieren
	rcall PLL_DELAY
	cbi PORTB,pll_ena
	rcall PLL_DELAY
	ret

PLL_DELAY:						;verbraucht 8 maschinentakte (2s@4MHz)
        nop
        ret
;==================================================================================

SLEEP_25ms:						;25 millisekunden warten
    ldi var1,0xff   
SLP25_2:
    ldi var2,0x82
SLP25_1:
    dec var2
    brne SLP25_1
    dec var1
    brne SLP25_2
    ret